﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Query;
using System.Xml.XLinq;
using System.Data.DLinq;
using System.Collections;
using System.ComponentModel;
using System.IO;

namespace LINQQueries
{
    class Program
    {
        static void Main(string[] args)
        {
            //WhereExtenstionMethod();
            //WhereAndSelectExtenstionMethod();
            //SimplifyWithLambdaAndVar();
            //SimplifyWithLinq();
            //ProjectingAResult();
            //ProjectingAnAnonymousType();
            //ProjectingAnAnonymousType();
            //WorkingWithAnonymousTypes();
            //MultipleSources();
            //UsingJoins();
            //NestedQueries();
            //OrderBy();
            //DeferredExecution();
            //QueryDirectories();
       }

        #region 1: Example showing Where extension method against customers

        private static void WhereExtenstionMethod()
        {
            DataSource ds = new DataSource();
            List<Customer> customers = ds.GetCustomerList();

            IEnumerable<Customer> results = customers.Where(delegate(Customer c) { return c.City == "Seattle"; });

            ObjectDumper.Write(results);
        }
        #endregion

        #region 2: Example showing Where and Select extension method against customers

        private static void WhereAndSelectExtenstionMethod()
        {
            DataSource ds = new DataSource();
            List<Customer> customers = ds.GetCustomerList();

            //Compiler can't infer the arguments on the select so we need to be explicit instead of using an anon. method
            Func<Customer, string> selector = new Func<Customer, string>(Selector);

            //Note that we are now IEnumerable<string> because type comes from select clause!
            IEnumerable<string> results = 
                customers
                .Where(delegate(Customer c) { return c.City == "Seattle"; })
                .Select(selector);

            ObjectDumper.Write(results);
        }

        private static string Selector(Customer c)
        {
            return c.CompanyName;
        }

        #endregion

        #region 3: Showing how lamda and var can simplify a query expression
        private static void SimplifyWithLambdaAndVar()
        {
            DataSource ds = new DataSource();
            List<Customer> customers = ds.GetCustomerList();

            //var just indicates that the compiler should infer the type; the lamda expression is a more convenient syntax for a delegate
            var results =
                customers
                .Where(c => c.City == "Seattle")
                .Select(c => c.CompanyName);

            ObjectDumper.Write(results);
        }
        #endregion

        #region 4: Shows how the LINQ syntax cleans this up even further
        private static void SimplifyWithLinq()
        {
            DataSource ds = new DataSource();
            List<Customer> customers = ds.GetCustomerList();

            //the LINQ syntax allows us to elide the fact that we our using extension methods and lambdas
            var query =
                from c in customers
                where c.City == "Seattle"
                select c.CompanyName;

            ObjectDumper.Write(query);
         }
        #endregion

        #region 5: Shows how to project a result
        private static void ProjectingAResult()
        {
            DataSource ds = new DataSource();
            List<Customer> customers = ds.GetCustomerList();

            //We can project from our full type into a restricted result set
            var query =
                 from c in customers
                 where c.City == "Seattle"
                 select new Contact(c.CustomerID, c.CompanyName);

            ObjectDumper.Write(query);
        }
        #endregion

        #region 6: Shows projection with anonymous types
        private static void ProjectingAnAnonymousType()
        {
           DataSource ds = new DataSource();
            List<Customer> customers = ds.GetCustomerList();

            //An anoymous type takes the pain out of projection and prevents 'type explosion'
            var results =
                 from c in customers
                 where c.City == "Seattle"
                 select new {c.CustomerID, c.CompanyName};

            ObjectDumper.Write(results);        
        }
        #endregion

        #region 7 : Shows how we work with an anonymous type
        private static void WorkingWithAnonymousTypes()
        {
            DataSource ds = new DataSource();
            List<Customer> customers = ds.GetCustomerList();

            //we can rename the fields of an anonymous type from thier default
            var results =
                 from c in customers
                 where c.City == "Seattle"
                 select new {Key = c.CustomerID, Description = c.CompanyName};

            //We can refer to the instance in a collection using 'var' too because the type is anon.
            foreach (var result in results)
            {
                Console.WriteLine(string.Format("Key: {0}  Description: {1}", result.Key, result.Description));
            }
        }

        //[Description("Will not compile")]
        //private void ShowResults(var results)
        //{
        //    foreach (var result in results)
        //    {
        //        Console.WriteLine(string.Format("Key: {0}  Description: {1}", result.Key, result.Description));
        //    }
        //}

        #endregion

        #region 8 : MultipleSources
        private static void MultipleSources()
        {
            DataSource ds = new DataSource();
            List<Customer> customers = ds.GetCustomerList();

            var query =
                    from c in customers
                    from o in c.Orders
                    where o.OrderDate.Year >= 1998
                    select new {c.CustomerID, o.OrderID, o.OrderDate};

            ObjectDumper.Write(query);

        }
        #endregion

        #region 9: using a join to produce the intersection of two sets
        private static void UsingJoins()
        {
            DataSource ds = new DataSource();
            List<Customer> customers = ds.GetCustomerList();

            Contact[] keyContacts = new Contact[]{
                { Key = "ALFKI", Name = "Homer Simpson" },
                { Key = "ANATR", Name = "Fred Flintstone" }
            };

            var query =
                    from k in keyContacts
                    join c in customers on k.Key equals c.CustomerID
                    select new { c.CustomerID, k.Name};

            ObjectDumper.Write(query);
        }
        #endregion

        #region 10: Using a nested query
        private static void NestedQueries()
        {
            DataSource ds = new DataSource();
            List<Customer> customers = ds.GetCustomerList();

            //We can use nested queries where an IEnumerable is expected
            var query =
                 from c in customers
                 where c.City == "Seattle"
                 select new {c.CompanyName, Orders=from o in c.Orders where o.OrderDate.Year == 1997 select new {o.OrderDate, o.Total}};

            ObjectDumper.Write(query, 1);
        }
        #endregion

        #region 11: An order by clause can be used to sort results
        private static void OrderBy()
        {
            DataSource ds = new DataSource();
            List<Customer> customers = ds.GetCustomerList();

            //We can order queries
            var query =
                 from c in customers
                 where c.City == "London"
                 orderby c.Address
                 select new {c.CompanyName, c.Address, Orders=from o in c.Orders where o.OrderDate.Year == 1997 select new {o.OrderDate, o.Total}};

            ObjectDumper.Write(query, 1);        }
        #endregion

        #region 12: Show that we are building a query not a result set; result set appears when we enumerate
        private static void DeferredExecution()
        {
            DataSource ds = new DataSource();
            List<Customer> customers = ds.GetCustomerList();

            //We can compose a query, it is not executed until we enumerate it
            var query =
                from c in customers
                where c.Country == "USA"
                select new {c.CompanyName, c.City, c.PostalCode, c.Country};

            var narrowed = 
                from c in query
                where c.City == "Seattle"
                select  new {c.CompanyName};

            ObjectDumper.Write(narrowed);
        }
        #endregion

        #region 13: We can query anything that supports an IEnumerable
        private static void QueryDirectories()
        {
            DirectoryInfo di = new DirectoryInfo(@"C:\Users\Ian Cooper\Documents\work");

            var query =
                from d in di.GetDirectories("*", SearchOption.AllDirectories)
                orderby d.Name
                select d.FullName;

            ObjectDumper.Write(query);
        }
        #endregion
    }
}
